// Copyright (c)1995 Ray Dream, Inc. All Rights Reserved.
/*$Id: LinkFac.cpp 1.1 1996/07/19 00:05:30 Damien Exp $*/

#ifndef __LINKFAC__
#include "LinkFac.h"
#endif

#ifndef __COMLINK__
#include "COMLink.h"
#endif

#ifndef __LINKDLL__
#include "Linkdll.h"
#endif


// ***** ClassFactory *****
  
ScrewLinkClassFactory::ScrewLinkClassFactory() {
  m_cRef=0L;  // just created so no reference used
  return;
  }


ScrewLinkClassFactory::~ScrewLinkClassFactory() {
  return;
  }

// IUnknown methods of ScrewLinkClassFactory                                                
STDMETHODIMP ScrewLinkClassFactory::QueryInterface(REFIID riid, LPVOID* ppv) {
  *ppv=NULL;

  if (IsEqualIID(riid, IID_IUnknown) || IsEqualIID(riid, IID_IClassFactory))
    *ppv=(LPVOID)this;

  if (*ppv!=NULL) {
    ((LPUNKNOWN)*ppv)->AddRef(); // Add reference if we give a pointer
    return NOERROR;
    }
  else {
    return ResultFromScode(E_NOINTERFACE);
    }
  }


STDMETHODIMP_(ULONG) ScrewLinkClassFactory::AddRef() {
  return ++m_cRef;
  }


STDMETHODIMP_(ULONG) ScrewLinkClassFactory::Release() {
  ULONG   UnreleaseRef;

  UnreleaseRef=--m_cRef;

  if (m_cRef == 0)
    delete this; // No reference left, so delete the ScrewLinkClassFactory

  return UnreleaseRef;
  }

/*
 * ScrewLinkClassFactory::CreateInstance
 *
 * Parameters:
 *  pUnkOuter       LPUNKNOWN to the controlling IUnknown if we are
 *                  being used in an aggregation.
 *  riid            REFIID identifying the interface the caller
 *                  desires to have for the new object.
 *  ppvObj          LPVOID FAR* in which to store the desired
 *                  interface pointer for the new object.
 *
 * Return Value:
 *  HRESULT         NOERROR if successful, otherwise E_NOINTERFACE
 *                  if we cannot support the requested interface.
 */

STDMETHODIMP ScrewLinkClassFactory::CreateInstance(LPUNKNOWN pUnkOuter, REFIID riid, LPVOID FAR* ppvObj) {
  ScrewLink*      pObj = NULL;
  HRESULT       hr;

  *ppvObj = NULL;
  hr = ResultFromScode(E_OUTOFMEMORY);

  //Verify that a controlling unknown asks for IUnknown
  if (pUnkOuter && !IsEqualIID(riid, IID_IUnknown))
    return ResultFromScode(E_NOINTERFACE);

  //Create the object passing function to notify on destruction.
  if (IsEqualIID(riid, IID_IUnknown) || IsEqualIID(riid, IID_I3DExMotionLink))
    pObj = new ScrewLink;
  else
    return ResultFromScode(E_NOINTERFACE);

  hr=pObj->QueryInterface(IID_I3DExMotionLink, ppvObj);

  //Kill the object if initial creation failed.
  if (FAILED(hr))
    delete pObj;
  else
    global_count_Obj++;

  return hr;
  }


/*
 * ScrewLinkClassFactory::LockServer
 *
 * Purpose:
 *  Increments or decrements the lock count of the DLL.  If the
 *  lock count goes to zero and there are no objects, the DLL
 *  is allowed to unload.  See DllCanUnloadNow in "ScrewLinkdll.cpp".
 *
 * Parameters:
 *  fLock           BOOL specifying whether to increment or
 *                  decrement the lock count.
 *
 * Return Value:
 *  HRESULT         NOERROR always.
 */

STDMETHODIMP ScrewLinkClassFactory::LockServer(BOOL fLock) {
  if (fLock)
      global_count_Lock++;
  else
      global_count_Lock--;

  return NOERROR;
  }

